/**
 * Copyright 2010 Trustees of the University of Pennsylvania Licensed under the
 * Educational Community License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License. You may
 * obtain a copy of the License at
 *
 * http://www.osedu.org/licenses/ECL-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS"
 * BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package metridoc.dsl

import metridoc.utils.Assert

import org.slf4j.LoggerFactory

/**
 *
 * @author tbarker
 */
class JobManager {

    def jobs = [:]
    def JobRunner jobRunner
    def Map<String, Object> services = [:]
    def ClassLoader jobClassLoader = Thread.currentThread().contextClassLoader
    def log = LoggerFactory.getLogger(JobManager.class)
    def String metridocDirectory = System.getProperty("user.home") + "/.metridoc"
    def boolean firstPassComplete = false

    def void parseJobs(String... jobFiles) {
        jobFiles.each {

            def fileName = it
            if(!fileName.endsWith(".groovy")) {
                fileName += ".groovy"
            }

            def stream = jobClassLoader.getResourceAsStream(fileName)
            def jobsContainer
            if (stream != null) {
                def reader = new InputStreamReader(jobClassLoader.getResourceAsStream(fileName))
                def shell = new GroovyShell()
                jobsContainer = shell.parse(reader, fileName)
            } else {
                try {
                    def clazz = jobClassLoader.loadClass(it)
                    jobsContainer = clazz.newInstance()
                } catch (ClassNotFoundException ex) {
                    throw new IllegalArgumentException("Could not find job ${it} as a file or class")
                }
            }

            jobsContainer.metaClass.jobs = {
                this.loadJobs(it, false)
            }

            jobsContainer.run()
        }

        firstPassComplete = true
    }

    void loadJobs(Closure closure, firstPassComplete = true) {
        closure.delegate = this
        closure.setDirective(Closure.DELEGATE_ONLY)
        closure()
        this.firstPassComplete = firstPassComplete
    }

    void job(String name, Closure closure) {
        jobs[name] = closure
    }

    void runJob(String jobName) {
        log.info "beginning job ${jobName}"
        Assert.isTrue jobs.containsKey(jobName), "the job ${jobName} cannot be run since it has not been loaded"
        def Closure closure = jobs[jobName]

        if (getJobRunner().exceptionOccurred) {
            def message = "an exception occurred running a previous job, cannot execute ${jobName}.  " +
                    "Please look at previous log items for details."
            log.error message
        } else {
            getJobRunner().run closure
            if (getJobRunner().exceptionOccurred) {
                def message = "an exception occurred while running ${jobName}.  " +
                        "Please look at previous log items for details."
                log.error message
            }

        }

    }

    def getJobRunner() {
        if (jobRunner == null) {
            jobRunner = new JobRunner(manager: this)
        }

        return jobRunner
    }

    def stop() {
        jobRunner.camelContext.stop()
        jobRunner.camelContext = null
        jobRunner.producerTemplate = null
    }

    def methodMissing(String name, args) {
        Assert.notEmpty(args, "method ${name} is not legal in job dsl with args ${args}")

        if (firstPassComplete) {
            throw new IllegalArgumentException("method ${name} with args ${args} is not allowed in the job dsl")
        }

        Assert.isTrue(args[0] instanceof Closure, "args must have one argument and be of type Closure")
        def closure = args[0]

        Assert.isTrue(args.size() == 1, "args must have one argument and be of type Closure")
        job name, args[0]
    }
}

