/**
 * Copyright 2010 Trustees of the University of Pennsylvania Licensed under the
 * Educational Community License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License. You may
 * obtain a copy of the License at
 *
 * http://www.osedu.org/licenses/ECL-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS"
 * BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

package metridoc.processor;

import java.util.List;
import java.util.Map;
import java.util.concurrent.locks.ReentrantLock;
import metridoc.utils.CollectionUtils.Lists;
import org.apache.camel.Exchange;
import org.apache.camel.Processor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author Tommy Barker
 */
public class SimpleAggregatorProcessor implements Processor{

    private enum Status{processing, done};
    private int size = 500;
    private long timeout = 200;
    private List body = Lists.createArrayList();
    private Map headers;
    private ReentrantLock lock = new ReentrantLock(true);
    private String aggregateId;
    public static final String AGGREGATION_ID = "Metridoc.Aggregation.Id";
    private static final Logger log = LoggerFactory.getLogger(SimpleAggregatorProcessor.class);

    public SimpleAggregatorProcessor() {
    }

    public SimpleAggregatorProcessor(int size, long timeout) {
        this.size = size;
        this.timeout = timeout;
    }

    @Override
    public void process(Exchange exchange) throws Exception {
        doAggregation(exchange);
    }

    public void doAggregation(Exchange exchange) throws InterruptedException {
        
        final String idToSend;
        final List bodyToSend;
        final Map headersToSend;

        try {
            lock.lock();
            if (body.size() >= size) {
                log.debug("aggregation for id {} is complete since body is >= {}", aggregateId, size);
                aggregateId = null;
            }
            if (aggregateId == null) {
                aggregateId = exchange.getExchangeId();
                body = Lists.createArrayList();
                headers = exchange.getIn().getHeaders();
            }

        } finally {
            lock.unlock();
        }
        
        idToSend = aggregateId;
        bodyToSend = body;
        headersToSend = headers;

        bodyToSend.add(exchange.getIn().getBody());

        exchange.getOut().setHeaders(headersToSend);
        exchange.getOut().setBody(bodyToSend);
        exchange.getOut().setHeader(AGGREGATION_ID, idToSend);

        if (bodyToSend.size() < size) {
            Thread.sleep(timeout);
        }
        
        if (idToSend.equals(aggregateId)) {
            try {
                lock.lock();
                log.debug("aggregation for id {} is complete since timeout occurred with timeout {}", aggregateId, 
                        timeout);
                if (idToSend.equals(aggregateId)) {
                    aggregateId = null;
                }
            } finally {
                lock.unlock();
            }
        }
    }
}
